@@ -50,6 +50,7 @@ which needs to be better organized via functions */
 
 #include "camel-ews-folder.h"
 #include "camel-ews-private.h"
+#include "camel-ews-search.h"
 #include "camel-ews-store.h"
 #include "camel-ews-summary.h"
 #include "camel-ews-utils.h"
@@ -835,6 +836,7 @@ ews_folder_search_by_expression (CamelFolder *folder,
                                  GError **error)
 {
 	CamelEwsFolder *ews_folder;
+	CamelEwsSearch *ews_search;
 	CamelEwsFolderPrivate *priv;
 	GPtrArray *matches;
 
@@ -843,9 +845,15 @@ ews_folder_search_by_expression (CamelFolder *folder,
 
 	g_mutex_lock (&priv->search_lock);
 
+	ews_search = CAMEL_EWS_SEARCH (ews_folder->search);
+
 	camel_folder_search_set_folder (ews_folder->search, folder);
+	camel_ews_search_set_cancellable_and_error (ews_search, cancellable, error);
+
 	matches = camel_folder_search_search (ews_folder->search, expression, NULL, cancellable, error);
 
+	camel_ews_search_set_cancellable_and_error (ews_search, NULL, NULL);
+
 	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
@@ -858,6 +866,7 @@ ews_folder_count_by_expression (CamelFolder *folder,
                                 GError **error)
 {
 	CamelEwsFolder *ews_folder;
+	CamelEwsSearch *ews_search;
 	CamelEwsFolderPrivate *priv;
 	guint32 matches;
 
@@ -866,9 +875,15 @@ ews_folder_count_by_expression (CamelFolder *folder,
 
 	g_mutex_lock (&priv->search_lock);
 
+	ews_search = CAMEL_EWS_SEARCH (ews_folder->search);
+
 	camel_folder_search_set_folder (ews_folder->search, folder);
+	camel_ews_search_set_cancellable_and_error (ews_search, cancellable, error);
+
 	matches = camel_folder_search_count (ews_folder->search, expression, cancellable, error);
 
+	camel_ews_search_set_cancellable_and_error (ews_search, NULL, NULL);
+
 	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
@@ -882,6 +897,7 @@ ews_folder_search_by_uids (CamelFolder *folder,
                            GError **error)
 {
 	CamelEwsFolder *ews_folder;
+	CamelEwsSearch *ews_search;
 	CamelEwsFolderPrivate *priv;
 	GPtrArray *matches;
 
@@ -893,9 +909,15 @@ ews_folder_search_by_uids (CamelFolder *folder,
 
 	g_mutex_lock (&priv->search_lock);
 
+	ews_search = CAMEL_EWS_SEARCH (ews_folder->search);
+
 	camel_folder_search_set_folder (ews_folder->search, folder);
+	camel_ews_search_set_cancellable_and_error (ews_search, cancellable, error);
+
 	matches = camel_folder_search_search (ews_folder->search, expression, uids, cancellable, error);
 
+	camel_ews_search_set_cancellable_and_error (ews_search, NULL, NULL);
+
 	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
@@ -1491,6 +1513,40 @@ camel_ews_folder_new (CamelStore *store,
 		return NULL;
 	}
 
+	if (camel_offline_folder_can_downsync (CAMEL_OFFLINE_FOLDER (folder))) {
+		CamelSettings *settings;
+		gboolean offline_limit_by_age = FALSE;
+		CamelTimeUnit offline_limit_unit;
+		gint offline_limit_value;
+		time_t when = (time_t) 0;
+
+		settings = camel_service_ref_settings (CAMEL_SERVICE (store));
+
+		g_object_get (
+			settings,
+			"limit-by-age", &offline_limit_by_age,
+			"limit-unit", &offline_limit_unit,
+			"limit-value", &offline_limit_value,
+			NULL);
+
+		g_clear_object (&settings);
+
+		if (offline_limit_by_age)
+			when = camel_time_value_apply (when, offline_limit_unit, offline_limit_value);
+
+		if (when <= (time_t) 0)
+			when = (time_t) -1;
+
+		/* Ensure cache will expire when set up, otherwise
+		 * it causes redownload of messages too soon. */
+		camel_data_cache_set_expire_age (ews_folder->cache, when);
+		camel_data_cache_set_expire_access (ews_folder->cache, when);
+	} else {
+		/* Set cache expiration for one week. */
+		camel_data_cache_set_expire_age (ews_folder->cache, 60 * 60 * 24 * 7);
+		camel_data_cache_set_expire_access (ews_folder->cache, 60 * 60 * 24 * 7);
+	}
+
 	if (!g_ascii_strcasecmp (folder_name, "Inbox") ||
 	    folder_has_inbox_type (CAMEL_EWS_STORE (store), folder_name)) {
 		CamelSettings *settings;
@@ -1507,7 +1563,7 @@ camel_ews_folder_new (CamelStore *store,
 		g_object_unref (settings);
 	}
 
-	ews_folder->search = camel_folder_search_new ();
+	ews_folder->search = camel_ews_search_new (CAMEL_EWS_STORE (store));
 	if (!ews_folder->search) {
 		g_object_unref (folder);
 		return NULL;
@@ -1891,12 +1947,16 @@ ews_refresh_info_sync (CamelFolder *folder,
 	do {
 		GSList *items_created = NULL, *items_updated = NULL;
 		GSList *items_deleted = NULL;
+		gchar *new_sync_state = NULL;
 		guint32 total, unread;
 
 		e_ews_connection_sync_folder_items_sync (cnc, EWS_PRIORITY_MEDIUM, sync_state, id, "IdOnly", NULL, EWS_MAX_FETCH_COUNT,
-			&sync_state, &includes_last_item, &items_created, &items_updated, &items_deleted,
+			&new_sync_state, &includes_last_item, &items_created, &items_updated, &items_deleted,
 			cancellable, &local_error);
 
+		g_free (sync_state);
+		sync_state = new_sync_state;
+
 		if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
 			g_clear_error (&local_error);
 			camel_ews_summary_set_sync_state (CAMEL_EWS_SUMMARY (folder_summary), NULL);
