@@ -16,9 +16,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "evolution-ews-config.h"
 
 #include "e-mail-config-ews-backend.h"
 
@@ -215,7 +213,7 @@ mail_config_ews_backend_insert_widgets (EMailConfigServiceBackend *backend,
 		}
 	}
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		settings, "use-impersonation",
 		widget, "active",
 		G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
@@ -231,7 +229,7 @@ mail_config_ews_backend_insert_widgets (EMailConfigServiceBackend *backend,
 		widget, "text",
 		G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		settings, "use-impersonation",
 		widget, "sensitive",
 		G_BINDING_SYNC_CREATE);
@@ -240,7 +238,7 @@ mail_config_ews_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	gtk_grid_attach (GTK_GRID (container), widget, 2, 4, 1, 1);
 	gtk_widget_show (widget);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		priv->impersonate_user_entry, "sensitive",
 		widget, "sensitive",
 		G_BINDING_SYNC_CREATE);
@@ -284,7 +282,7 @@ mail_config_ews_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	/* Don't use G_BINDING_SYNC_CREATE here since the widget
 	 * chooses its initial mechanism more intelligently than
 	 * a simple property binding would. */
-	g_object_bind_property (
+	e_binding_bind_property (
 		settings, "auth-mechanism",
 		priv->auth_check, "active-mechanism",
 		G_BINDING_BIDIRECTIONAL);
@@ -294,7 +292,7 @@ mail_config_ews_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	extension = e_source_get_extension (source, extension_name);
 
 	/* The collection identity is the user name. */
-	g_object_bind_property (
+	e_binding_bind_property (
 		settings, "user",
 		extension, "identity",
 		G_BINDING_BIDIRECTIONAL |
@@ -355,12 +353,15 @@ static gboolean
 mail_config_ews_backend_check_complete (EMailConfigServiceBackend *backend)
 {
 	EMailConfigServicePage *page;
+	EMailConfigEwsBackendPrivate *priv;
 	CamelSettings *settings;
 	CamelEwsSettings *ews_settings;
 	CamelNetworkSettings *network_settings;
 	const gchar *hosturl;
 	const gchar *user;
+	gboolean correct, complete = TRUE;
 
+	priv = E_MAIL_CONFIG_EWS_BACKEND_GET_PRIVATE (backend);
 	page = e_mail_config_service_backend_get_page (backend);
 
 	/* This backend serves double duty.  One instance holds the
@@ -381,13 +382,17 @@ mail_config_ews_backend_check_complete (EMailConfigServiceBackend *backend)
 	network_settings = CAMEL_NETWORK_SETTINGS (settings);
 	user = camel_network_settings_get_user (network_settings);
 
-	if (hosturl == NULL || *hosturl == '\0')
-		return FALSE;
+	correct = hosturl != NULL && *hosturl != '\0';
+	complete = complete && correct;
 
-	if (user == NULL || *user == '\0')
-		return FALSE;
+	e_util_set_entry_issue_hint (priv->host_entry, correct ? NULL : _("Host URL cannot be empty"));
 
-	return TRUE;
+	correct = user != NULL && *user != '\0';
+	complete = complete && correct;
+
+	e_util_set_entry_issue_hint (priv->user_entry, correct ? NULL : _("User name cannot be empty"));
+
+	return complete;
 }
 
 static void
